cwlVersion: v1.0
class: CommandLineTool


requirements:
- class: ShellCommandRequirement
- class: InlineJavascriptRequirement
  expressionLib:
  - var get_output_filename = function(ext) {
        ext = ext || "";
        if (inputs.output_filename == null){
          let root = inputs.rsem_isoforms_file.basename.split('.').slice(0,-1).join('.');
          return (root == "")?inputs.rsem_isoforms_file.basename+ext:root+ext;
        } else {
          return inputs.output_filename;
        }
    };


hints:
- class: DockerRequirement
  dockerPull: biowardrobe2/scidap:v0.0.3


inputs:

  script:
    type: string?
    default: |
      #!/usr/bin/env python
      import sys
      opts = {
          2: lambda l : l.strip(),
          3: lambda l : l.strip().split()[0],
          4: lambda l : [str(int(l.strip().split()[1])-1), l.strip().split()[-1]]
      }
      print "RefseqId,GeneId,Chrom,TxStart,TxEnd,Strand,TotalReads,Rpkm"
      with open(sys.argv[1], 'r') as iso_stream, open(sys.argv[2], 'r') as anno_stream:
          iso_stream.readline()
          anno_stream.readline()
          for iso_line in iso_stream:
              iso_list = iso_line.strip().split('\t')
              result = []
              for idx in range(6):
                  anno_line = anno_stream.readline().strip()
                  if idx in opts.keys(): result.append(opts[idx](anno_line))
              print ','.join( [iso_list[0],iso_list[1],result[0],result[2][0],result[2][1],result[1],iso_list[4],iso_list[6]])
    inputBinding:
      position: 5
    doc: |
      Python script to generate BioWardrobe compatible isforoms file from RSEM outputs

  rsem_isoforms_file:
    type: File
    inputBinding:
      position: 6
    doc: |
      Generated by RSEM isoform file

  rsem_annotation_file:
    type: File
    inputBinding:
      position: 7
    doc: |
      *.ti file from RSEM indices folder to include sorted information about processed isoforms

  output_filename:
    type: string?
    doc: |
      Name for output file


outputs:

  biowardrobe_isoforms_file:
    type: File
    outputBinding:
      glob: "*"


baseCommand: [python, '-c']
arguments:
  - valueFrom: $(" > " + get_output_filename(".csv"))
    position: 100000
    shellQuote: false
